@@ -466,8 +466,9 @@ set_unlock_options_on_object (GkmWrapPrompt *self, CK_ATTRIBUTE_PTR options, CK_
 
 	rv = (self->module->C_SetAttributeValue) (self->session, self->object, &attr, 1);
 	if (rv != CKR_OK && rv != CKR_ATTRIBUTE_TYPE_INVALID) {
-		g_warning ("Couldn't set credential template for prompt: %s",
-		           gkm_util_rv_to_string (rv));
+		if (rv != CKR_TOKEN_WRITE_PROTECTED)
+			g_warning ("Couldn't set credential template for prompt: %s",
+			           gkm_util_rv_to_string (rv));
 	}
 }
 
@@ -604,7 +605,7 @@ prepare_unlock_keyring_login (GkmWrapPrompt *self)
 
 	gku_prompt_set_title (prompt, _("Unlock Login Keyring"));
 
-	text = _("Enter password for to unlock your login keyring");
+	text = _("Enter password to unlock your login keyring");
 	gku_prompt_set_primary_text (prompt, text);
 
 	if (gkm_wrap_login_did_unlock_fail ())
